currentGamemode = nil
currentGamemodeMap = nil
setGameType(false)
setMapName("None")

rootElement = getRootElement()
local serverConsole = getElementByIndex("console", 0)

addEventHandler("onResourceStop", rootElement, 
	function (stoppedResource)
		local resourceRoot = getResourceRootElement(stoppedResource)
		if stoppedResource == currentGamemode then
			triggerEvent("onGamemodeStop", resourceRoot, currentGamemode)
		elseif stoppedResource == currentGamemodeMap then
			triggerEvent("onGamemodeMapStop", resourceRoot, currentGamemodeMap)
		end
	end
)

addEvent("onGamemodeStart")
addEvent("onGamemodeStop")
addEvent("onGamemodeMapStart")
addEvent("onGamemodeMapStop")

addEventHandler("onGamemodeStart", rootElement, 
	function ( startedGamemode )
		currentGamemode = startedGamemode
		local gamemodeName = getResourceInfo(startedGamemode, "name") or getResourceName(startedGamemode)
		
		if get("ASE") then
			setGameType(gamemodeName)
		end
		if get("messages") then
			outputMapManager("Gamemode '"..gamemodeName.."' started.")
		end
	end
)

addEventHandler("onGamemodeMapStart", rootElement, 
	function ( startedGamemodeMap )
		currentGamemodeMap = startedGamemodeMap
		local gamemodeMapName = getResourceInfo(startedGamemodeMap, "name") or getResourceName(startedGamemodeMap)
		
		applyMapSettings( currentGamemodeMap )
		
		if get("ASE") then
			setMapName(gamemodeMapName)
		end
		if get("messages") then
			outputMapManager("Map '"..gamemodeMapName.."' started.")
		end
	end
)

addEventHandler("onGamemodeStop", rootElement, 
	function ()
		currentGamemode = nil
		if currentGamemodeMap then
			stopResource(currentGamemodeMap)
		end
		
		setGameType(false)
	end
)

addEventHandler("onGamemodeMapStop", rootElement, 
	function ()
		currentGamemodeMap = nil
		
		resetMapInfo()		
		
		setMapName("None")
	end
)

local function changeGamemodeMap_cmd(source, command, mapName, gamemodeName)
	source = source or serverConsole

	local map
	if mapName then
		map = getResourceFromName(mapName)
		if not isMap(map) then
			outputMapManager("'"..mapName.."' is not a valid map.",source)
			return false
		end
	else
		outputMapManager("Usage: /"..command.." map [gamemode]",source)
		return false
	end
	
	local gamemode = currentGamemode
	if gamemodeName then
		gamemode = getResourceFromName(gamemodeName)
		if not isGamemode(gamemode) then
			outputMapManager("'"..gamemodeName.."' is not a valid gamemode.",source)
			return false
		end
	end

	if not isGamemode(gamemode) then
		outputMapManager("No gamemode is running.",source)
	else
		changeGamemodeMap(map, gamemode)
	end
end
addCommandHandler("changemap", changeGamemodeMap_cmd, true)

local function changeGamemode_cmd(source, command, gamemodeName, mapName)
	source = source or serverConsole

	local gamemode
	if gamemodeName then
		gamemode = getResourceFromName(gamemodeName)
		if not isGamemode(gamemode) then
			outputMapManager("'"..gamemodeName.."' is not a valid gamemode.",source)
			return false
		end
	else
		outputMapManager("Usage: /"..command.." gamemode [map]",source)
		return false
	end
	
	local map
	if mapName then
		map = getResourceFromName(mapName)
		if not isMap(map) then
			outputMapManager("'"..mapName.."' is not a valid map.",source)
			return false
		end
	end
	
	changeGamemode(gamemode,map)
end
addCommandHandler("gamemode", changeGamemode_cmd, true)
addCommandHandler("changemode", changeGamemode_cmd, true)

local function stopGamemode_cmd(source)
	source = source or serverConsole
	
	local runningMode = getRunningGamemode()
	if runningMode then
		stopGamemode()
		outputMapManager("Gamemode '"..getResourceName(runningMode).."' stopped.",source)
	else
		outputMapManager("No gamemode is running.",source)
	end
end
addCommandHandler("stopmode", stopGamemode_cmd)

local function stopGamemodeMap_cmd(source)
	source = source or serverConsole
	
	local runningMap = getRunningGamemodeMap()
	if runningMap then
		stopGamemodeMap()
		outputMapManager("Map '"..getResourceName(runningMap).."' stopped.",source)
	else
		outputMapManager("No gamemode map is running.",source)
	end
end
addCommandHandler("stopmap", stopGamemodeMap_cmd)

local function outputGamemodeListToConsole(source)
	source = source or serverConsole

	local allGamemodes = getGamemodes()
	local numberOfGamemodes = #allGamemodes

	if numberOfGamemodes == 0 then
		outputMapManagerConsole("There are no gamemodes.", source)
	else
		local s = "s"
		if numberOfGamemodes == 1 then s="" end
		outputMapManagerConsole("There are "..numberOfGamemodes.." gamemode"..s..":", source)
	end

	for k, gamemode in ipairs(allGamemodes) do
		local gamemodeFriendlyName = getResourceInfo(gamemode, "name")
		if gamemodeFriendlyName then
			gamemodeFriendlyName = " ("..gamemodeFriendlyName..") "
		else
			gamemodeFriendlyName = ""
		end
		
		local numberOfCompatibleMaps = #getMapsCompatibleWithGamemode(gamemode)
		
		local s = "s"
		if numberOfCompatibleMaps == 1 then s="" end
		
		outputMapManagerConsole(getResourceName(gamemode) .. gamemodeFriendlyName .. " [".. numberOfCompatibleMaps .. " map"..s.."]", source)
	end
end
addCommandHandler("gamemodes",outputGamemodeListToConsole)

local function outputMapListToConsole(source, command, gamemodeName)
	source = source or serverConsole

	if not gamemodeName then
		local allMaps = getMaps()
		local numberOfMaps = #allMaps
		
		if numberOfMaps == 0 then
			outputMapManagerConsole("There are no maps.", source)
		else
			local s = "s"
			if numberOfMaps == 1 then s="" end
			outputMapManagerConsole("There are "..numberOfMaps.." map"..s..":", source)
		end
		
		for k, map in ipairs(allMaps) do
			local gamemodeMapFriendlyName = getResourceInfo(map, "name")
			if gamemodeMapFriendlyName then
				gamemodeMapFriendlyName = " ("..gamemodeMapFriendlyName..") "
			else
				gamemodeMapFriendlyName = ""
			end
			outputMapManagerConsole(getResourceName(map) .. gamemodeMapFriendlyName, source)
		end
	else
		local gamemode = getResourceFromName(gamemodeName)
		if not gamemode then
			outputMapManager("Gamemode '"..gamemodeName.."' doesn't exist.", source)
			return false
		end
		
		local compatibleMaps = getMapsCompatibleWithGamemode(gamemode)
		if not compatibleMaps then
			outputMapManager("Gamemode '"..gamemodeName.."' doesn't exist.", source)
			return false
		end
		
		local numberOfCompatibleMaps = #compatibleMaps
		if numberOfCompatibleMaps == 0 then
			outputMapManagerConsole("'"..gamemodeName.."' has no maps.", source)
		else
			local s = "s"
			if numberOfCompatibleMaps == 1 then s="" end
			outputMapManagerConsole("'"..gamemodeName.."' has "..#compatibleMaps.." map"..s..":", source)
		end
		
		for k, map in ipairs(compatibleMaps) do
			local gamemodeMapFriendlyName = getResourceInfo(map, "name")
			if gamemodeMapFriendlyName then
				gamemodeMapFriendlyName = " ("..gamemodeMapFriendlyName..") "
			else
				gamemodeMapFriendlyName = ""
			end
			outputMapManagerConsole(getResourceName(map) .. gamemodeMapFriendlyName, source)
		end
	end
end
addCommandHandler("maps",outputMapListToConsole)

function outputMapManager(message, toElement)
	toElement = toElement or rootElement
	local r, g, b = getColorFromString(string.upper(get("color")))
	if getElementType(toElement) == "console" then
		outputServerLog(message)
	else
		outputChatBox(message, toElement, r, g, b)
		if toElement == rootElement then
			outputServerLog(message)
		end
	end
end

function outputMapManagerConsole(message, toElement)
	toElement = toElement or rootElement
	if getElementType(toElement) == "console" then
		outputServerLog(message)
	else
		outputConsole(message, toElement)
		if toElement == rootElement then
			outputServerLog(message)
		end
	end
end

function doesMapSupportPlayerCount( map )
	local mapName = getResourceName(map)
	local minPlayers = get(mapName..".minplayers")
	local minPlayers = get(mapName..".maxplayers")
	
	local playersIn = getPlayerCount()
	
	if minPlayers and minPlayers < playersIn then
		outputMapManager( "More than "..(minPlayers-1).." are required to start '"..mapName.."'" )
		return false
	end
	
	if maxPlayers and maxPlayers > playersIn then
		outputMapManager( "Less than "..(maxPlayers+1).." are required to start '"..mapName.."'" )
		return false
	end
		
	return true
end

local hr, mn

local settingApplier = {
	gamespeed = function(value) setGameSpeed(tonumber(value)) end,
	gravity = function(value) setGravity(tonumber(value)) end,
	time = function(value)
		local splitString = split(value, string.byte(':'))
		hr = tonumber(splitString[1]) or 12
		mn = tonumber(splitString[2]) or 0
		setTime(hr, mn)
	end,
	weather = function(value) setWeather(tonumber(value)) end,
	waveheight = function(value) setWaveHeight(tonumber(value)) end,
}

local defaultSettings = {
	gamespeed = 1,
	gravity = 0.008,
	time = "12:00",
	weather = 0,
	waveheight = 0,
}

local function freezeTimeOnJoin()
	triggerClientEvent(source, "mm.doFreezeTime", source, hr, mn)
end

local function removeTimeFreeze()
	local hr, mn = getTime()
	triggerClientEvent(rootElement, "mm.doUnfreezeTime", rootElement, hr, mn)
	removeEventHandler("onPlayerJoin", rootElement, freezeTimeOnJoin)
end
addEventHandler("onGamemodeMapStop", rootElement, removeTimeFreeze)

function applyMapSettings( map )
	local mapSettingsGroup = getResourceName(map).."."
	for setting, defaultValue in pairs(defaultSettings) do
		settingApplier[setting](get(mapSettingsGroup..setting) or defaultValue)
	end
	
	if get(mapSettingsGroup.."locked_time") then
		triggerClientEvent(rootElement, "mm.doFreezeTime", rootElement, hr, mn)
		addEventHandler("onPlayerJoin", rootElement, freezeTimeOnJoin)
	end
end
