g_Root = getRootElement()
g_ResRoot = getResourceRootElement(getThisResource())
g_Me = getLocalPlayer()
g_ArmedVehicleIDs = table.create({ 425, 447, 520, 430, 464, 432 }, true)

g_Checkpoints = {}
g_Pickups = {}
g_VisiblePickups = {}
g_Objects = {}

addEventHandler('onClientResourceStart', g_ResRoot,
	function()
		g_Players = getElementsByType('player')
		
		-- create GUI
		local screenWidth, screenHeight = guiGetScreenSize()
		g_GUI = {
			rankbg = guiCreateStaticImage(screenWidth-160, screenHeight-138, 160, 138, 'rankbg.png', false, nil),
			rank = guiCreateLabel(screenWidth-155, screenHeight-133, 150, 20, '', false),
			checkpoint = guiCreateLabel(screenWidth-155, screenHeight-98, 150, 20, '', false),
			timepassed = guiCreateLabel(screenWidth-155, screenHeight-52, 150, 20, '', false),
			timeleftbg = guiCreateStaticImage(screenWidth/2-108/2, 15, 108, 24, 'timeleft.png', false, nil),
			timeleft = guiCreateLabel(screenWidth/2-108/2, 19, 108, 30, '', false),
			healthbar = FancyProgress.create(100, 1000, 'progress_health_bg.png', screenWidth-187, 105, 123, 30, 'progress_health.png', 7, 7, 109, 16),
			speedbar = FancyProgress.create(0, 1.5, 'progress_speed_bg.png', screenWidth-187, 135, 123, 30, 'progress_speed.png', 7, 7, 109, 16)
		}
		guiSetAlpha(g_GUI.rankbg, 0.3)
		guiSetVisible(g_GUI.rankbg, false)
		g_GUI.healthbar:hide()
		g_GUI.speedbar:hide()
		for i,name in ipairs({'rank', 'checkpoint', 'timepassed', 'timeleft'}) do
			guiSetFont(g_GUI[name], 'default-bold-small')
		end
		guiLabelSetHorizontalAlign(g_GUI.timeleft, 'center')
		g_GUI.speedbar:setProgress(0)
		
		g_PickupStartTick = getTickCount()
		addEventHandler('onClientRender', g_Root, updateBars)
		g_WaterCheckTimer = setTimer(checkWater, 1000, 0)
		
		-- load pickup models and textures
		local txd, dff
		for name,id in pairs({nitro = 1337, repair = 1338, carchange = 1339}) do
			txd = engineLoadTXD(name .. '.txd')
			engineImportTXD(txd, id)
			dff = engineLoadDFF(name .. '.dff')
			engineReplaceModel(dff, id)
		end
		
		setPlayerCanBeKnockedOffBike(g_Me, false)
		triggerServerEvent('onLoadedAtClient', g_Me)
	end
)

function initRace(vehicle, checkpoints, objects, pickups, duration)
	fadeCamera(true)
	showHUD(false)
	
	g_Vehicle = vehicle
	
	-- checkpoints
	if g_Checkpoints then
		for i=1,#g_Checkpoints do
			destroyCheckpoint(i)
		end
	end
	g_Checkpoints = checkpoints
	
	-- pickups
	local pickupElem
	local pos
	local colshape
	local modelForPickupType = { nitro = 1337, repair = 1338, vehiclechange = 1339 }
	for i,pickup in pairs(pickups) do
		pos = pickup.position
		object = createObject(modelForPickupType[pickup.type], pos[1], pos[2], pos[3])
		setElementCollisionsEnabled(object, false)
		colshape = createColTube(pos[1]-1, pos[2]-1, pos[3]-3, 2, 6)
		g_Pickups[colshape] = { object = object }
		for k,v in pairs(pickup) do
			g_Pickups[colshape][k] = v
		end
	end
	
	-- objects
	g_Objects = {}
	local pos, rot
	for i,object in ipairs(objects) do
		pos = object.position
		rot = object.rotation
		g_Objects[i] = createObject(object.model, pos[1], pos[2], pos[3], rot[1], rot[2], rot[3])
	end
	
	if #g_Checkpoints > 0 then
		g_CurrentCheckpoint = 0
		showNextCheckpoint()
		updateRank()
		showGUIComponents('rank', 'checkpoint')
	else
		hideGUIComponents('rank', 'checkpoint')
	end
	
	g_GUI.healthbar:show()
	g_GUI.speedbar:show()
	showGUIComponents('rankbg')
	
	if duration then
		launchRace(duration)
	end
end

function launchRace(duration, vehicleWeapons)
	if type(duration) == 'number' then
		showGUIComponents('timepassed', 'timeleftbg', 'timeleft')
		guiLabelSetColor(g_GUI.timeleft, 255, 255, 255)
		g_Duration = duration*1000
		addEventHandler('onClientRender', g_Root, updateTime)
	else
		hideGUIComponents('timepassed', 'timeleftbg', 'timeleft')
	end
	setVehicleFrozen(g_Vehicle, false)
	if not vehicleWeapons and g_ArmedVehicleIDs[getVehicleID(g_Vehicle)] then
		toggleControl('vehicle_fire', false)
		toggleControl('vehicle_secondary_fire', false)
	end
	g_StartTick = getTickCount()
	if #g_Checkpoints > 0 then
		g_RankTimer = setTimer(updateRank, 1000, 0)
	end
end

addEventHandler('onClientElementStreamIn', g_Root,
	function()
		local colshape = table.find(g_Pickups, 'object', source)
		if colshape then
			local pickup = g_Pickups[colshape]
			if pickup.type == 'vehiclechange' then
				pickup.label = guiCreateLabel(0, 0, 100, 20, getVehicleNameFromID(pickup.vehicle), false)
				guiSetVisible(pickup.label, false)
				guiLabelSetHorizontalAlign(pickup.label, 'center')
			end
			g_VisiblePickups[colshape] = source
			g_VisiblePickups.n = (g_VisiblePickups.n or 0) + 1
			if g_VisiblePickups.n == 1 then
				addEventHandler('onClientRender', g_Root, updatePickups)
			end
		end
	end
)

addEventHandler('onClientElementStreamOut', g_Root,
	function()
		local colshape = table.find(g_VisiblePickups, source)
		if colshape then
			local pickup = g_Pickups[colshape]
			if pickup.label then
				destroyElement(pickup.label)
				pickup.label = nil
				pickup.labelVisible = nil
			end
			g_VisiblePickups[colshape] = nil
			g_VisiblePickups.n = g_VisiblePickups.n - 1
			if g_VisiblePickups.n == 0 then
				removeEventHandler('onClientRender', g_Root, updatePickups)
			end
		end
	end
)

function updatePickups()
	local angle = math.fmod((getTickCount() - g_PickupStartTick) * 360 / 2000, 360)
	local g_Pickups = g_Pickups
	local pickup, x, y, pX, pY, pZ
	for colshape,elem in pairs(g_VisiblePickups) do
		if colshape ~= 'n' then
			setObjectRotation(elem, 0, 0, angle)
			pickup = g_Pickups[colshape]
			if pickup.label then
				local pX, pY, pZ = getElementPosition(g_Me)
				x, y = getScreenFromWorldPosition(unpack(pickup.position))
				if x and getDistanceBetweenPoints3D(pX, pY, pZ, unpack(pickup.position)) < 80 then
					if not pickup.labelVisible then
						guiSetVisible(pickup.label, true)
						pickup.labelVisible = true
					end
					guiSetPosition(pickup.label, x-50, y-50, false)
				else
					if pickup.labelVisible then
						guiSetVisible(pickup.label, false)
						pickup.labelVisible = false
					end
				end
			end
		end
	end
end

addEventHandler('onClientColShapeHit', g_Root,
	function(elem)
		local pickup = g_Pickups[source]
		if elem ~= g_Vehicle or not pickup then
			return
		end
		triggerServerEvent('onPlayerPickUpRacePickup', g_Me, pickup.id)
		playSoundFrontEnd(46)
		
		if pickup.type == 'vehiclechange' then
			setTimer(checkNewVehicleZ, 1000, 1)
		end
	end
)

function checkNewVehicleZ()
	local x, y, z = getElementPosition(g_Vehicle)
	local groundZ = getGroundPosition(x, y, z+10) + getElementDistanceFromCentreOfMassToBaseOfModel(g_Vehicle)
	if z < groundZ then
		setElementPosition(g_Vehicle, x, y, groundZ)
	end
end

function vehicleUnloading()
	g_Vehicle = nil
end

function updateBars()
	if g_Vehicle then
		g_GUI.healthbar:setProgress(getElementHealth(g_Vehicle))
		local vx, vy, vz = getElementVelocity(g_Vehicle)
		g_GUI.speedbar:setProgress(math.sqrt(vx*vx + vy*vy + vz*vz))
	end
end

function updateTime()
	local tick = getTickCount()
	local msPassed = tick - g_StartTick
	if not isPlayerFinished(g_Me) then
		guiSetText(g_GUI.timepassed, msToTimeStr(msPassed))
	end
	local timeLeft
	if g_HurryDuration then
		timeLeft = g_HurryDuration - (tick - g_HurryStartTick)
	else
		timeLeft = g_Duration - msPassed
	end
	guiSetText(g_GUI.timeleft, msToTimeStr(timeLeft))
end

function updateRank()
	local rank = getMyRank()
	guiSetText(g_GUI.rank, rank .. ' / ' .. #g_Players)
	setElementData(g_Me, 'Race rank', rank)
end

function checkWater()
	local x, y, z = getElementPosition(g_Me)
	local waterZ = getWaterLevel(x, y, z)
	if waterZ and z < waterZ - 0.5 and not isPlayerFinished(g_Me) then
		setElementHealth(g_Me, 0)
	end
end

function msToTimeStr(ms)
	if not ms then
		return ''
	end
	local centiseconds = tostring(math.floor(math.fmod(ms, 1000)/10))
	if #centiseconds == 1 then
		centiseconds = '0' .. centiseconds
	end
	local s = math.floor(ms / 1000)
	local seconds = tostring(math.fmod(s, 60))
	if #seconds == 1 then
		seconds = '0' .. seconds
	end
	local minutes = tostring(math.floor(s / 60))
	return minutes .. ':' .. seconds .. ':' .. centiseconds
end

function showNextCheckpoint()
	g_CurrentCheckpoint = g_CurrentCheckpoint + 1
	local i = g_CurrentCheckpoint
	guiSetText(g_GUI.checkpoint, (i-1) .. ' / ' .. #g_Checkpoints)
	if i > 1 then
		destroyCheckpoint(i-1)
	else
		createCheckpoint(1)
	end
	makeCheckpointCurrent(i)
	if i < #g_Checkpoints then
		local curCheckpoint = g_Checkpoints[i]
		local nextCheckpoint = g_Checkpoints[i+1]
		local nextMarker = createCheckpoint(i+1)
		setMarkerTarget(curCheckpoint.marker, unpack(nextCheckpoint.position))
	end
	setElementData(g_Me, 'race.checkpoint', i)
end

function checkpointReached(elem)
	if elem ~= g_Vehicle then
		return
	end
	triggerServerEvent('onPlayerReachCheckpoint', g_Me, g_CurrentCheckpoint, g_Duration and (getTickCount() - g_StartTick))
	playSoundFrontEnd(43)
	if g_CurrentCheckpoint < #g_Checkpoints then
		showNextCheckpoint()
	else
		guiSetText(g_GUI.checkpoint, #g_Checkpoints .. ' / ' .. #g_Checkpoints)
		if g_RankTimer then
			killTimer(g_RankTimer)
			g_RankTimer = false
		end
		if g_GUI.hurry then
			destroyElement(g_GUI.hurry)
			g_GUI.hurry = nil
		end
		setElementData(g_Me, 'race.finished', true)
		destroyCheckpoint(#g_Checkpoints)
		toggleAllControls(false, true, false)
	end
end

function getMyRank()
	local rank = 1
	local checkpoint
	
	for i,player in ipairs(g_Players) do
		if player ~= g_Me then
			checkpoint = getElementData(player, 'race.checkpoint') or 1
			if isPlayerFinished(player) == true or checkpoint > g_CurrentCheckpoint then
				rank = rank + 1
			elseif checkpoint == g_CurrentCheckpoint then
				if distanceFromPlayerToCheckpoint(player, checkpoint) < distanceFromPlayerToCheckpoint(g_Me, checkpoint) then
					rank = rank + 1
				end
			end
		end
	end
	return rank
end

function distanceFromPlayerToCheckpoint(player, i)
	local checkpoint = g_Checkpoints[i]
	local x, y, z = getElementPosition(player)
	return getDistanceBetweenPoints3D(x, y, z, unpack(checkpoint.position))
end

function startHurry(timeLeft)
	if not isPlayerFinished(g_Me) then
		local screenWidth, screenHeight = guiGetScreenSize()
		g_GUI.hurry = guiCreateStaticImage(screenWidth/2 - 364/2, screenHeight - 120, 364, 82, 'hurry.png', false, nil)
		Animation.createAndPlay(g_GUI.hurry, Animation.presets.pulse(1000))
	end
	guiLabelSetColor(g_GUI.timeleft, 255, 0, 0)
	g_HurryDuration = timeLeft
	g_HurryStartTick = getTickCount()
end

function startSpectate()
	if #g_Players == 1 then
		return
	end
	local screenWidth, screenHeight = guiGetScreenSize()
	g_GUI.specprev = guiCreateStaticImage(screenWidth/2 - 120 - 33, screenHeight - 120, 34, 65, 'specprev.png', false, nil)
	g_GUI.specnext = guiCreateStaticImage(screenWidth/2 + 120, screenHeight - 120, 34, 65, 'specnext.png', false, nil)
	g_GUI.speclabel = guiCreateLabel(screenWidth/2 - 100, screenHeight - 100, 200, 50, '', false)
	guiLabelSetHorizontalAlign(g_GUI.speclabel, 'center')
	repeat
		g_SpectatedPlayer = g_Players[math.random(#g_Players)]
	until g_SpectatedPlayer ~= g_Me
	bindKey('arrow_l', 'down', spectatePrevious)
	bindKey('arrow_r', 'down', spectateNext)
	updateSpectate()
end

function spectatePrevious()
	local i = table.find(g_Players, g_SpectatedPlayer)
	local startI = i
	repeat
		i = i == 1 and #g_Players or (i - 1)
	until (g_Players[i] ~= g_Me and not isPlayerFinished(g_Players[i])) or i == startI
	if i == startI then
		i = i == 1 and #g_Players or (i - 1)
	end
	g_SpectatedPlayer = g_Players[i]
	updateSpectate()
end

function spectateNext()
	local i = table.find(g_Players, g_SpectatedPlayer)
	local startI = i
	repeat
		i = i == #g_Players and 1 or (i + 1)
	until (g_Players[i] ~= g_Me and not isPlayerFinished(g_Players[i])) or i == startI
	if i == startI then
		i = i == #g_Players and 1 or (i + 1)
	end
	g_SpectatedPlayer = g_Players[i]
	updateSpectate()
end

function updateSpectate()
	setCameraTarget(g_SpectatedPlayer)
	guiSetText(g_GUI.speclabel, 'Currently spectating:\n' .. getPlayerName(g_SpectatedPlayer))
end

function stopSpectate()
	for i,name in ipairs({'specprev', 'specnext', 'speclabel'}) do
		if g_GUI[name] then
			destroyElement(g_GUI[name])
			g_GUI[name] = nil
		end
	end
	unbindKey('arrow_l', 'down', spectatePrevious)
	unbindKey('arrow_r', 'down', spectateNext)
	setCameraTarget(g_Me)
	g_SpectatedPlayer = nil
end

function raceTimeout()
	removeEventHandler('onClientRender', g_Root, updateTime)
	destroyCheckpoint(g_CurrentCheckpoint)
	destroyCheckpoint(g_CurrentCheckpoint + 1)
	guiSetText(g_GUI.timeleft, msToTimeStr(0))
	guiSetText(g_GUI.timepassed, msToTimeStr(g_Duration))
	if g_GUI.hurry then
		destroyElement(g_GUI.hurry)
		g_GUI.hurry = nil
	end
	toggleAllControls(false, true, false)
end

function endRace()
	if #g_Checkpoints > 0 then
		destroyCheckpoint(g_CurrentCheckpoint)
		if g_CurrentCheckpoint < #g_Checkpoints then
			destroyCheckpoint(g_CurrentCheckpoint+1)
		end
		g_Checkpoints = {}
		g_CurrentCheckpoint = nil
	end
	
	for colshape,pickup in pairs(g_Pickups) do
		destroyElement(colshape)
		destroyElement(pickup.object)
	end
	g_Pickups = {}
	g_VisiblePickups = {}
	removeEventHandler('onClientRender', g_Root, updatePickups)
	
	table.each(g_Objects, destroyElement)
	g_Objects = {}
	
	setElementData(g_Me, 'race.checkpoint', nil)
	
	g_Vehicle = nil
	removeEventHandler('onClientRender', g_Root, updateTime)
	if g_RankTimer then
		killTimer(g_RankTimer)
		g_RankTimer = false
	end
	
	toggleAllControls(false, true, false)
	
	if g_GUI then
		hideGUIComponents('rankbg', 'rank', 'checkpoint', 'timepassed', 'timeleftbg', 'timeleft')
		g_GUI.healthbar:hide()
		g_GUI.speedbar:hide()
		
		if g_GUI.hurry then
			destroyElement(g_GUI.hurry)
			g_GUI.hurry = nil
		end
	end
	g_HurryDuration = nil
	g_HurryStartTick = nil
	if g_SpectatedPlayer then
		stopSpectate()
	end
end

function createCheckpoint(i)
	local checkpoint = g_Checkpoints[i]
	if checkpoint.marker then
		return
	end
	local pos = checkpoint.position
	local color = checkpoint.color or { 255, 0, 0 }
	checkpoint.size = checkpoint.size or 2.25
	checkpoint.marker = createMarker(pos[1], pos[2], pos[3], checkpoint.type or 'checkpoint', checkpoint.size*2, color[1], color[2], color[3])
	if (not checkpoint.type or checkpoint.type == 'checkpoint') and i == #g_Checkpoints then
		setMarkerIcon(checkpoint.marker, 'finish')
	end
	checkpoint.blip = createBlip(pos[1], pos[2], pos[3], 0, isCurrent and 2 or 1, color[1], color[2], color[3])
	return checkpoint.marker
end

function makeCheckpointCurrent(i)
	local checkpoint = g_Checkpoints[i]
	local pos = checkpoint.position
	local color = checkpoint.color or { 255, 0, 0 }
	if not checkpoint.blip then
		checkpoint.blip = createBlip(pos[1], pos[2], pos[3], 0, 2, color[1], color[2], color[3])
	else
		setBlipSize(checkpoint.blip, 2)
	end
	
	if not checkpoint.type or checkpoint.type == 'checkpoint' then
		checkpoint.colshape = createColCircle(pos[1], pos[2], checkpoint.size*4)
	else
		checkpoint.colshape = createColSphere(pos[1], pos[2], pos[3], checkpoint.size*8)
	end
	addEventHandler('onClientColShapeHit', checkpoint.colshape, checkpointReached, false)
end

function destroyCheckpoint(i)
	local checkpoint = g_Checkpoints[i]
	if checkpoint and checkpoint.marker then
		destroyElement(checkpoint.marker)
		checkpoint.marker = nil
		destroyElement(checkpoint.blip)
		checkpoint.blip = nil
		if checkpoint.colshape then
			destroyElement(checkpoint.colshape)
			checkpoint.colshape = nil
		end
	end
end

function setCurrentCheckpoint(i)
	destroyCheckpoint(g_CurrentCheckpoint)
	destroyCheckpoint(g_CurrentCheckpoint + 1)
	createCheckpoint(i)
	g_CurrentCheckpoint = i - 1
	showNextCheckpoint()
end

function isPlayerFinished(player)
	return getElementData(player, 'race.finished')
end

addEvent('onClientCall', true)
addEventHandler('onClientCall', g_Root,
	function(fnName, ...)
		local fn = _G
		local path = fnName:split('.')
		for i,pathpart in ipairs(path) do
			fn = fn[pathpart]
		end
		fn(...)
	end
)

addEventHandler('onClientPlayerJoin', g_Root,
	function()
		table.insert(g_Players, source)
	end
)

addEventHandler('onClientPlayerQuit', g_Root,
	function()
		table.removevalue(g_Players, source)
		if source == g_SpectatedPlayer then
			if #g_Players > 1 then
				spectateNext()
			else
				stopSpectate()
			end
		end
	end
)

addEventHandler('onClientResourceStop', g_ResRoot,
	function()
		removeEventHandler('onClientRender', g_Root, updateBars)
		removeEventHandler('onClientRender', g_Root, updatePickups)
		killTimer(g_WaterCheckTimer)
		toggleAllControls(true)
		showHUD(true)
		setPlayerCanBeKnockedOffBike(g_Me, true)
	end
)
