g_Root = getRootElement()
g_ResRoot = getResourceRootElement(getThisResource())
scoreboard = createResourceCallInterface('scoreboard')
votemanager = createResourceCallInterface('votemanager')
g_MotorBikeIDs = table.create({ 448, 461, 462, 463, 468, 471, 521, 522, 523, 581, 586 }, true)
g_ArmedVehicleIDs = table.create({ 425, 447, 520, 430, 464, 432 }, true)
g_RCVehicles = table.create({ 441, 464, 465, 501, 564 }, true)

g_Spawnpoints = {}			-- { i = { position={x, y, z}, rotation=rotation, vehicle=vehicleID, paintjob=paintjob, upgrades={...} } }
g_Checkpoints = {}			-- { i = { position={x, y, z}, size=size, color={r, g, b}, type=type, vehicle=vehicleID, paintjob=paintjob, upgrades={...} } }
g_Objects = {}				-- { i = { position={x, y, z}, rotation={x, y, z}, model=modelID } }
g_Pickups = {}				-- { i = { position={x, y, z}, type=type, vehicle=vehicleID, paintjob=paintjob, upgrades={...} }

g_PlayerSkins = {}			-- { player = skin }
g_Vehicles = {}				-- { player = vehicle }
g_CheckpointBackups = {}	-- { player = { goingback = true/false, i = { vehicle = id, position = {x, y, z}, rotation = {x, y, z}, velocity = {x, y, z} } } }

--[[
do
	local aclgroup = aclGetGroup('raceACLGroup')
	if not aclgroup then
		aclgroup = aclCreateGroup('raceACLGroup')
		aclGroupAddObject(aclgroup, 'resource.race')
	end
	
	local acl = aclGet('raceACL')
	if not acl then
		acl = aclCreate('raceACL')
		aclGroupAddACL(aclgroup, acl)
	end
	aclSetRight(acl, 'general.ModifyOtherObjects', true)
	
	aclSave()
end
--]]

function loadMap(res)
	local map = RaceMap.load(res)
	if not map then
		outputDebugString('Error loading map')
		return false
	end
	
	-- set options
	if map:isRaceFormat() then
		if map.time then
			setTime(map.time:match('(%d+):(%d+)'))
		end
		if map.weather then
			setWeather(map.weather)
		end
	end
	g_Duration = map.duration or 1800
	g_RespawnType = map.respawn or 'timelimit'
	g_RespawnTime = map.respawntime and map.respawntime*1000 or 5000
	g_SkinType = map.skins or 'cj'
	if g_SkinType == 'cj' then
		g_BikeHats = map.bikehats or '12 13'
		g_BikeHatChance = map.bikehatchance or 1
		g_CarHats = map.carhats or '16-20 22-31 34 36-40 43-47'
		g_CarHatChance = map.carhatchance or 0.6
		g_HairStyles = map.hairstyles or '0-30 32'
		g_Glasses = map.glasses or '9-15'
		g_GlassesChance = map.glasseschance or 1
		g_Shirts = map.shirts or '1 3 4 7-21 23 25-41 43-60'
		g_Trousers = map.trousers or '1-4 7 10-14 28 29 34 36 37 43'
		g_Shoes = map.shoes or '2-5 6 10-30 33'
	end
	g_VehicleWeapons = map.vehicleweapons == 'true'
	g_GhostMode = map.ghostmode == 'true'
	
	-- read spawnpoints
	g_Spawnpoints = map:getAll('spawnpoint')
	--[[
	for i,spawnpoint in ipairs(g_Spawnpoints) do
		spawnpoint.rotation = 360 - spawnpoint.rotation
	end
	--]]
	
	-- read checkpoints
	g_Checkpoints = map:getAll('checkpoint')
	if map:isDMFormat() then
		-- sort checkpoints
		local chains = {}		-- a chain is a list of checkpoints that immediately follow each other
		local prevchainnum, chainnum, nextchainnum
		for i,checkpoint in ipairs(g_Checkpoints) do
			-- is it the finish?
			if not checkpoint.nextid then
				table.insert(chains, { checkpoint })
			else
				-- any chain we can place this checkpoint after?
				chainnum = table.find(chains, '[last]', 'nextid', checkpoint.id)
				if chainnum then
					table.insert(chains[chainnum], checkpoint)
					nextchainnum = table.find(chains, 1, 'id', checkpoint.nextid)
					if nextchainnum then
						table.merge(chains[chainnum], chains[nextchainnum])
						table.remove(chains, nextchainnum)
					end
				else
					-- any chain we can place it before?
					chainnum = table.find(chains, 1, 'id', checkpoint.nextid)
					if chainnum then
						table.insert(chains[chainnum], 1, checkpoint)
						prevchainnum = table.find(chains, '[last]', 'nextid', checkpoint.id)
						if prevchainnum then
							table.merge(chains[prevchainnum], chains[chainnum])
							table.remove(chains, chainnum)
						end
					else
						-- new chain
						table.insert(chains, { checkpoint })
					end
				end
			end
		end
		g_Checkpoints = chains[1]
	end
	
	-- read objects
	g_Objects = map:getAll('object')
	
	-- read pickups
	g_Pickups = map:getAll('pickup')
	
	-- unload map xml
	map:unload()
	return true
end

function launchRace()
	clientCall(g_Root, 'launchRace', g_Duration, g_VehicleWeapons)
	if g_Duration then
		g_StartTick = getTickCount()
		g_RaceEndTimer = setTimer(raceTimeout, g_Duration*1000, 1)
	end
end
g_RaceStartCountdown = Countdown.create(3, launchRace)
--g_RaceStartCountdown:setBackground('countdown_bkg.png', 474, 204)
g_RaceStartCountdown:useImages('countdown_%d.png', 474, 204)
g_RaceStartCountdown:enableFade(true)
g_RaceStartCountdown:addClientHook(3, 'playSoundFrontEnd', 44)
g_RaceStartCountdown:addClientHook(2, 'playSoundFrontEnd', 44)
g_RaceStartCountdown:addClientHook(1, 'playSoundFrontEnd', 44)
g_RaceStartCountdown:addClientHook(0, 'playSoundFrontEnd', 45)

addEventHandler('onGamemodeMapStart', g_Root,
	function(mapres)
		if not loadMap(mapres) then
			return
		end
		startRace()
	end
)

function startRace(fastStart)
	g_StartTick = false
	for i,player in ipairs(getElementsByType('player')) do
		joinHandler(player)
	end
	setTimer(startCountdown, fastStart and 500 or 5000, 1)
end

function startCountdown()
	g_RaceStartCountdown:start()
end

function restartRace()
	for player,skin in pairs(g_PlayerSkins) do
		clientCall(player, 'vehicleUnloading')
		destroyElement(g_Vehicles[player])
		destroyBlipsAttachedTo(player)
	end
	startRace(true)
end

function joinHandler(player)
	if #g_Spawnpoints == 0 then
		-- start vote if no racetrack is loaded
		votemanager.voteMap(getThisResource())
		return
	end
	local playerJoined = not player
	if not player then
		player = source
	end
	
	local i
	if not table.find(g_Spawnpoints, 'used', '[nil]') then
		i = math.random(#g_Spawnpoints)
	else
		repeat
			i = math.random(#g_Spawnpoints)
		until not g_Spawnpoints[i].used
	end
	local spawnpoint = g_Spawnpoints[i]
	spawnpoint.used = true
	
	local x, y, z = unpack(spawnpoint.position)
	if g_SkinType == 'cj' then
		spawnPlayer(player, x + 4, y, z, 0, 0)
		g_PlayerSkins[player] = 0
		
		local id, texture, model, clothes
		clothes = {}
		if g_MotorBikeIDs[spawnpoint.vehicle] then
			if math.random() <= g_BikeHatChance then
				clothes[16] = getRandomFromRangeList(g_BikeHats)
			end
		else
			if math.random() <= g_CarHatChance then
				clothes[16] = getRandomFromRangeList(g_CarHats)
			end
		end
		if not clothes[16] then
			clothes[1] = getRandomFromRangeList(g_HairStyles)
		end
		if math.random() <= g_GlassesChance then
			clothes[15] = getRandomFromRangeList(g_Glasses)
		end
		clothes[0] = getRandomFromRangeList(g_Shirts)
		clothes[2] = getRandomFromRangeList(g_Trousers)
		clothes[3] = getRandomFromRangeList(g_Shoes)
		local texture, model
		for type,index in pairs(clothes) do
			texture, model = getClothesByTypeIndex(type, index)
			addPlayerClothes(player, texture, model, type)
		end
		for type=15,16 do
			if not clothes[type] then
				removePlayerClothes(player, type)
			end
		end
	elseif g_SkinType == 'random' then
		local skin, spawnSuccess
		repeat
			skin = math.random(9, 288)
			spawnSuccess = spawnPlayer(player, x + 4, y, z, 0, skin)
		until spawnSuccess
		g_PlayerSkins[player] = skin
	else
		local skin = getRandomFromRangeList(g_SkinType)
		g_PlayerSkins[player] = skin
		spawnPlayer(player, x + 4, y, z, 0, skin)
	end
	
	local vehicle = createVehicle(spawnpoint.vehicle, x, y, z, 0, 0, spawnpoint.rotation)
	setVehicleFrozen(vehicle, true)
	if playerJoined then
		setTimer(setVehicleFrozen, 3000, 1, vehicle, false)
	end
	if g_GhostMode then
		clientCall(g_Root, 'setElementCollisionsEnabled', vehicle, false)
	end
	setVehiclePaintjobAndUpgrades(vehicle, spawnpoint.paintjob, spawnpoint.upgrades)
	setElementHealth(vehicle, 1000)
	setTimer(warpPlayerIntoVehicle, 500, 1, player, vehicle)
	g_Vehicles[player] = vehicle
	
	setTimer(freeSpawnpoint, 2000, 1, i)
	if playerJoined then
		clientCall(player, 'initRace', vehicle, g_Checkpoints, g_Objects, g_Pickups, g_Duration and (g_Duration - (g_StartTick and math.floor((getTickCount() - g_StartTick)/1000) or 0)) or true)
	else
		clientCall(player, 'initRace', vehicle, g_Checkpoints, g_Objects, g_Pickups)
	end
	
	createBlipAttachedTo(player, 0, 1, 200, 200, 200)
	g_CheckpointBackups[player] = {}
	g_CheckpointBackups[player][0] = { vehicle = spawnpoint.vehicle, position = {x, y, z}, rotation = {0, 0, spawnpoint.rotation}, velocity = {0, 0, 0} }
	
	-- workaround for #2987 (player element data not sent to joining players)
	if playerJoined then
		for player,skin in pairs(g_PlayerSkins) do
			setElementData(player, 'race.checkpoint', getElementData(player, 'race.checkpoint'))
		end
	end
	
	if playerJoined and getPlayerCount() == 2 then
		-- Restart the race if someone joined a lone player
		local restartCountdown = Countdown.create(5, restartRace, 'Race will restart in:', 255, 255, 255)
		restartCountdown:start()
	end
end
addEventHandler('onPlayerJoin', g_Root, joinHandler)

function freeSpawnpoint(i)
	g_Spawnpoints[i].used = nil
end

addEvent('onPlayerReachCheckpoint', true)
addEventHandler('onPlayerReachCheckpoint', g_Root,
	function(checkpoint, time)
		if checkpoint < #g_Checkpoints then
			-- Regular checkpoint
			local vehicle = g_Vehicles[source]
			g_CheckpointBackups[source][checkpoint] = { vehicle = getVehicleID(vehicle), position = { getElementPosition(vehicle) }, rotation = { getVehicleRotation(vehicle) }, velocity = { getElementVelocity(vehicle) } }		
			local checkpoint = g_Checkpoints[checkpoint]
			if checkpoint.vehicle then
				setVehicleID(vehicle, checkpoint.vehicle)
				setVehicleColor(g_Vehicles[source], math.random(0, 126), math.random(0, 126), 0, 0)
			end
			setVehiclePaintjobAndUpgrades(vehicle, checkpoint.paintjob, checkpoint.upgrades)
			
			g_CheckpointBackups[source].goingback = true
			if g_CheckpointBackups[source].timer then
				killTimer(g_CheckpointBackups[source].timer)
			end
			g_CheckpointBackups[source].timer = setTimer(lastCheckpointWasSafe, 5000, 1, source)
		else
			-- Finish reached
			local rank = getElementData(source, 'Race rank')
			if rank == 1 then
				showMessage('You have won the race!', 0, 255, 0, source)
				g_RankingBoard = RankingBoard.create()
				if g_Duration then
					if getTickCount() - g_StartTick > 30000 then
						killTimer(g_RaceEndTimer)
						g_RaceEndTimer = setTimer(raceTimeout, 30000, 1)
						clientCall(g_Root, 'startHurry', 30000)
					else
						clientCall(g_Root, 'startHurry')
					end
				end
			else
				showMessage('You finished ' .. rank .. (({ [2] = 'nd', [3] = 'rd' })[rank] or 'th') .. '!', 0, 255, 0, source)
			end
			g_RankingBoard:add(source, time)
			if rank < getPlayerCount() then
				setTimer(clientCall, 5000, 1, source, 'startSpectate')
			else
				setTimer(endRace, 10000, 1)
			end
		end
	end
)

function lastCheckpointWasSafe(player)
	g_CheckpointBackups[player].goingback = false
	g_CheckpointBackups[player].timer = nil
end

addEventHandler('onPlayerWasted', g_Root,
	function()
		if g_CheckpointBackups[source].timer then
			killTimer(g_CheckpointBackups[source].timer)
			g_CheckpointBackups[source].timer = nil
		end
		if not g_GhostMode then
			clientCall(g_Root, 'setElementCollisionsEnabled', g_Vehicles[source], false)
		end
		if g_RespawnType == 'timelimit' and not isPlayerFinished(source) then
			local respawnCountdown = Countdown.create(g_RespawnTime/1000, restorePlayer, 'You will respawn in:', 255, 255, 255, source)
			respawnCountdown:start(source)
		elseif g_RespawnType == 'none' then
			if #g_Checkpoints == 0 then
				if not g_RankingBoard then
					g_RankingBoard = RankingBoard.create()
				end
				local alivePlayers = getAlivePlayers()
				g_RankingBoard:add(source, getTickCount() - g_StartTick, #alivePlayers + 1)
				if #alivePlayers == 1 then
					g_RankingBoard:add(alivePlayers[1], getTickCount() - g_StartTick, 1)
					showMessage(getClientName(alivePlayers[1]) .. ' is the final survivor!', 0, 255, 0)
					setTimer(endRace, 10000, 1)
					return
				end
			end
			setTimer(clientCall, 3000, 1, source, 'startSpectate')
		end
	end
)

function restorePlayer(player)
	local checkpoint = getPlayerCurrentCheckpoint(player)
	if g_CheckpointBackups[player].goingback and checkpoint > 1 then
		checkpoint = checkpoint - 1
		setPlayerCurrentCheckpoint(player, checkpoint)
	end
	g_CheckpointBackups[player].goingback = true
	local bkp = g_CheckpointBackups[player][checkpoint - 1]
	spawnPlayer(player, bkp.position[1], bkp.position[2], bkp.position[3], 0, g_PlayerSkins[player])

	local vehicle = g_Vehicles[player]
	fixVehicle(vehicle)
	setElementPosition(vehicle, unpack(bkp.position))
	setVehicleRotation(vehicle, unpack(bkp.rotation))
	if getVehicleID(vehicle) ~= bkp.vehicle then
		setVehicleID(vehicle, bkp.vehicle)
	end
	warpPlayerIntoVehicle(player, vehicle)
	
	if checkpoint > 1 then
		setVehicleFrozen(vehicle, true)
		toggleAllControls(player, false, true, false)
		setTimer(restorePlayerUnfreeze, 2000, 1, player)
	elseif not g_GhostMode then
		clientCall(g_Root, 'setElementCollisionsEnabled', g_Vehicles[player], true)
	end
end

function restorePlayerUnfreeze(player)
	toggleAllControls(player, true)
	if not g_VehicleWeapons and g_ArmedVehicleIDs[getVehicleID(g_Vehicles[player])] then
		toggleControl(player, 'vehicle_fire', false)
		toggleControl(player, 'vehicle_secondary_fire', false)
	end
	setVehicleFrozen(g_Vehicles[player], false)
	clientCall(g_Root, 'setElementCollisionsEnabled', g_Vehicles[player], true)
	setElementVelocity(g_Vehicles[player], unpack(g_CheckpointBackups[player][getPlayerCurrentCheckpoint(player)-1].velocity))
end

addEvent('onPlayerPickUpRacePickup', true)
addEventHandler('onPlayerPickUpRacePickup', g_Root,
	function(pickupID)
		local pickup = g_Pickups[table.find(g_Pickups, 'id', pickupID)]
		local vehicle = g_Vehicles[source]
		if pickup.type == 'repair' then
			fixVehicle(vehicle)
		elseif pickup.type == 'nitro' then
			addVehicleUpgrade(vehicle, 1010)
		elseif pickup.type == 'vehiclechange' then
			setVehicleID(vehicle, pickup.vehicle)
			setVehicleColor(vehicle, math.random(0, 126), math.random(0, 126), 0, 0)
			setVehiclePaintjobAndUpgrades(vehicle, pickup.paintjob, pickup.upgrades)
		end
	end
)

function raceTimeout()
	for player,skin in pairs(g_PlayerSkins) do
		if not isPlayerFinished(player) then
			showMessage('Time\'s up!')
		end
	end
	clientCall(g_Root, 'raceTimeout')
	g_RaceEndTimer = nil
	setTimer(endRace, 10000, 1)
end

function endRace(skipMapVote)
	if not skipMapVote then
		votemanager.voteMap(getThisResource())
	end
	triggerEvent('onGamemodeMapStop', g_Root)
end

function getPlayerCurrentCheckpoint(player)
	return getElementData(player, 'race.checkpoint') or 1
end

function setPlayerCurrentCheckpoint(player, i)
	clientCall(player, 'setCurrentCheckpoint', i)
end

function isPlayerFinished(player)
	return getElementData(player, 'race.finished')
end

addEventHandler('onGamemodeMapStop', g_Root,
	function(mapres)
		clientCall(g_Root, 'endRace')
		if g_RaceEndTimer then
			killTimer(g_RaceEndTimer)
			g_RaceEndTimer = nil
		end
		if g_RankingBoard then
			g_RankingBoard:destroy()
			g_RankingBoard = nil
		end

		for player,skin in pairs(g_PlayerSkins) do
			setElementData(player, 'race.finished', false)
		end
		
		table.each(g_Vehicles, destroyElement)
		g_Vehicles = {}
		g_Spawnpoints = {}
		g_Checkpoints = {}
		g_Objects = {}
		g_Pickups = {}
		g_CheckpointBackups = {}
	end
)

addEventHandler('onResourceStart', g_ResRoot,
	function()
		scoreboard.addScoreboardColumn('Race rank')
	end
)

addEventHandler('onResourceStop', g_ResRoot,
	function()
		scoreboard.removeScoreboardColumn('Race rank')
	end
)

addEventHandler('onPlayerQuit', g_Root,
	function()
		destroyBlipsAttachedTo(source)
		g_PlayerSkins[source] = nil
		if g_Vehicles[source] then
			destroyElement(g_Vehicles[source])
			g_Vehicles[source] = nil
		end
		g_CheckpointBackups[source] = nil
		for player,skin in pairs(g_PlayerSkins) do
			if not isPlayerFinished(player) then
				return
			end
		end
		setTimer(endRace, 5000, 1)
	end
)

addEventHandler('onVehicleDamage', g_Root,
	function()
		local player = table.find(g_Vehicles, source)
		if player then
			setElementHealth(player, getElementHealth(source)/10)
		end
	end
)

addEventHandler('onVehicleStartExit', g_Root, function() cancelEvent() end)

addCommandHandler('kill',
	function(player)
		killPlayer(player)
	end
)

addCommandHandler('convrace',
	function(player, command, resname)
		local map = RaceMap.load(getResourceFromName(resname))
		if not map then
			outputDebugString('Failed to load ' .. resname)
			return
		end
		if map:isDMFormat() then
			outputConsole('Map is already in deathmatch format.')
			return
		end
		map:convert()
		map:save()
		map:unload()
		outputConsole('Map converted successfully')
	end
)

addCommandHandler('ghostmode',
	function()
		g_GhostMode = not g_GhostMode
		for k,vehicle in pairs(g_Vehicles) do
			clientCall(g_Root, 'setElementCollisionsEnabled', vehicle, false)
		end
		outputConsole('Ghostmode is now ' .. (g_GhostMode and 'on' or 'off'))
	end
)